<?php declare(strict_types = 1);

namespace Drupal\view_transitions\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure View Transitions settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'view_transitions_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['view_transitions.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['page_transitions'] = [
      '#type' => 'radios',
      '#title' => $this->t('Full page view transitions'),
      '#default_value' => $this->config('view_transitions.settings')->get('page_transitions'),
      '#options' => [
        'off' => $this->t('Off'),
        'spa' => $this->t('Single-Page Application Mode'),
        'mpa' => $this->t('Multi-Page Application Mode (Experimental)'),
      ],
      'off' => [
        '#description' => $this->t('Disable full page view transitions.'),
      ],
      'spa' => [
        '#description' => $this->t('Enable full page view transitions using client-side routing.'),
      ],
      'mpa' => [
        '#description' => $this->t('Enable full page view transitions using server-side routing. Currently behind a flag in Chromium browsers.'),
      ],
    ];
    $form['animation_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Animation Type'),
      '#default_value' => $this->config('view_transitions.settings')->get('animation_type'),
      '#description' => $this->t('Non-default animations apply only to browsers that support view transitions.'),
      '#options' => [
        'default' => $this->t('Cross-fade (default)'),
        'slide' => $this->t('Slide (left to right)'),
        'circle' => $this->t('Circle'),
        'shutter' => $this->t('Shutter'),
        'wipe' => $this->t('Wipe (diagonal)'),
      ],
      '#states' => [
        // Show only if a page transition mode is enabled.
        'invisible' => [
          ':input[name="page_transitions"]' => ['value' => 'off'],
        ],
      ],
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // @todo Validate the form here.
    // Example:
    // @code
    //   if ($form_state->getValue('example') === 'wrong') {
    //     $form_state->setErrorByName(
    //       'message',
    //       $this->t('The value is not correct.'),
    //     );
    //   }
    // @endcode
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('view_transitions.settings')
      ->set('page_transitions', $form_state->getValue('page_transitions'))
      ->set('animation_type', $form_state->getValue('animation_type'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
